#!/usr/bin/lua

local libuci = require("uci")
local hardware_detection = require("lime.hardware_detection")
local config = require("lime.config")

local openwrt_wan = {}

openwrt_wan.sectionName = hardware_detection.sectionNamePrefix.."openwrt_wan"

function openwrt_wan.clean()
	if config.autogenerable(openwrt_wan.sectionName) then config.delete(openwrt_wan.sectionName) end
end

function openwrt_wan.detect_hardware()
	if config.autogenerable(openwrt_wan.sectionName) then
		local uci_o = libuci:cursor()
		uci_o:set_confdir("/etc/lime-config-firstboot")
		uci_o:set_savedir("/dev/null")
		local ifname = uci_o:get("network", "wan", "ifname")
		uci_o = nil
		if ifname then
			local protos = {}
			for _, proto in pairs(config.get("network", "protocols")) do
				if ( ( proto ~= "lan" ) and ( proto ~= "wan" ) ) then
					table.insert(protos, proto)
				end
			end
			table.insert(protos, "wan")

			config.init_batch()
			config.set(openwrt_wan.sectionName, "net")
			config.set(openwrt_wan.sectionName, "autogenerated", "true")
			config.set(openwrt_wan.sectionName, "protocols", protos)
			config.set(openwrt_wan.sectionName, "linux_name", ifname)
			config.end_batch()
		end
	end
end


return openwrt_wan
